/**************************************************************************************************
  Ethernet_Driver.c

**************************************************************************************************/

// Include Files
#include "Ethernet.h"
#include "__NetEthInternal.h"
#include "UART1.h"
#include "Buffers.h"
#include "recources.h"
#include "timers.h"
#include "CoreTimer.h"
#include "__Time.h"



// Constants
const unsigned long int Fudge = (((365 * 70) + 17) * 24 * 60 * 60); // Seconds from 1900 to 1970

// Variables

const code unsigned char httpHeader[] = "HTTP/1.1 200 OK\nContent-Length: 7787\nConnection: close\nContent-type: ";  // HTTP header
const code unsigned char httpMimeTypeHTML[] = "text/html\n\n";              // HTML MIME type
const code unsigned char httpImage[] = "HTTP/1.1 200 OK\nConnection: keep-alive\nContent-type: image/jpg\n\n";  // TEXT MIME type
unsigned char httpMethod[] = "GET /";
unsigned char httpRequ[] = "GET / HTTP/1.1";




// Variables used by Ethernet/Internet applications

// Variables used by all
//unsigned char myMacAddr[6] = {0x00,0x14,0xA5,0x76,0x19,0x3f};  // my MAC address
unsigned char myMacAddr[6];                       // My MAC address (using internal PIC MAC)
unsigned char myIpAddr[4]  = {192,168,2,4};       // My IP address
unsigned char gwIpAddr[4]  = {192,168,2,1};       // Gateway (router) IP address
unsigned char ipMask[4]    = {255,255,255,0};     // Network mask (for example : 255.255.255.0)
unsigned char dnsIpAddr[4] = {192,168,2,1};       // DNS server IP address

// Variables used by UDP server
//unsigned char dyna[32] ;                        // Buffer for dynamic response
char UDP_Message1[] = "Date/Time set";
char UDP_Message2[] = "NAK";
unsigned char UDP_Buffer[64];
extern TimeStruct ts1;
extern long Epoch;

// Variables used by SNTP server
unsigned char SNTP_Buffer[64];
unsigned char SNTP_TxBuffer[64];
struct SNTPFrame SNTPReceiveFrame;
struct SNTPFrame SNTPTransmitFrame;
extern unsigned long int msec;
extern long Epoch;



// Variables used by DHCP client
char Use_dhcp = FALSE;  // Set to TRUE if using DHCP (router)
char Got_dhcp = FALSE;

// Variables used by HTTP server
const unsigned int HTML_CODE_SIZE = 7501;
const unsigned int PICTURE1_SIZE = 46565;
const unsigned int PICTURE2_SIZE = 5168;
const unsigned int PICTURE3_SIZE = 4103;
const unsigned int PICTURE4_SIZE = 3778;

unsigned char getRequest[15];   // HTTP request buffer
char send_mark1 = 0, send_mark2 = 0, send_mark3 = 0, send_mark4 = 0, sendHTML_mark = 0;
SOCKET_Intern_Dsc *socketImage1, *socketImage2, *socketImage3, *socketImage4, *socketHTML;
unsigned int pos[7];  // was 5 VG


// Variables used by TCP client
SOCKET_Intern_Dsc *socketTCP_Client;  // Socket for TCP client
TCPC_FIFO_struct FIFO1;  // Outgoing message queue for TCP client
unsigned int TCPC_ClientState = TCPC_STATE_WAITING;  // Initial state of TCP client
unsigned char rmtIpAddr[4] = {192,168,2,5};   // IP address of remote TCP server
unsigned int rmtPort = 6000;   // Port number of remote TCP server we will connect to
unsigned int locPort = 6001;   // Local port TCP client will listen to
unsigned short int TCP_Timer;  // Timer for TCP client

// Variables used by TCP server
SOCKET_Intern_Dsc *socketTCP_Server;  // Socket for TCP server
unsigned char userData[128];



/*************************************************************************************************/
void InitPHYPins(){
     TRISD11_bit = 0;   // ETH_ALT_MDC_BIT
     TRISD8_bit = 1;    // ETH_ALT_MDIO_BIT

     TRISA15_bit = 0;   // ETH_ALT_TXEN_BIT
     TRISD14_bit = 0;   // ETH_ALT_TXD0_BIT
     TRISD15_bit = 0;   // ETH_ALT_TXD1_BIT

     TRISG9_bit = 1;    // ETH_ALT_RXCLK_BIT
     TRISG8_bit = 1;    // ETH_ALT_RXDV_BIT
     TRISE8_bit = 1;    // ETH_ALT_RXD0_BIT
     TRISE9_bit = 1;    // ETH_ALT_RXD1_BIT
     TRISG15_bit = 1;   // ETH_ALT_RXERR_BIT
}

/*************************************************************************************************/
void InitEthernet() {
     int i;
     unsigned int My[6];
     char *BufNum;
     
     InitPHYPins();  // Initialize physical pins
     
     Net_Ethernet_Intern_stackInitTCP();

      // Initialize Ethernet board
     Net_Ethernet_Intern_Init(myMacAddr, myIpAddr, Net_Eth_Int_AUTO_NEGOTIATION & Net_Eth_Int_DEFAULT_MAC );

     if (Use_dhcp) {  // These functions are blocking!!!
         if (Net_Ethernet_Intern_initDHCP(5)) { // Get network configuration from DHCP server, wait 5 sec for the response
             memcpy(myIpAddr,Net_Ethernet_Intern_getIpAddress(), 4); // Fetch IP address
             memcpy(gwIpAddr, Net_Ethernet_Intern_getGwIpAddress(), 4); // Fetch gateway IP address
             memcpy(dnsIpAddr, Net_Ethernet_Intern_getDnsIpAddress(), 4); // Fetch DNS server address
             memcpy(IpMask, Net_Ethernet_Intern_getIpMask(), 4); // Fetch IP subnet mask
             Got_dhcp = TRUE;
         
             Net_Ethernet_Intern_confNetwork(ipMask, gwIpAddr, dnsIpAddr);
         } else {
             UART1_putQueue("DHCP not responding\r\n");
             Got_dhcp = FALSE;
         }
     }
     Net_Ethernet_Intern_confNetwork(ipMask, gwIpAddr, dnsIpAddr);
     Delay_ms(1000);
     
     for (i = 0; i < NUM_OF_SOCKET_Intern; i++)
          pos[i] = 0;
     
     for (i=0;i<4;i++)
          My[i] = myIpAddr[i];
     BufNum = GetBuffer();
     sprintf(BufNum,"My IP: %d.%d.%d.%d\r\n",My[0],My[1],My[2],My[3]);
     UART1_PutQueue(BufNum);
}

/*************************************************************************************************/
void DoEthernet() {
     unsigned char Rtn;
     int i;
     
     Rtn = Net_Ethernet_Intern_doPacket();

/*
     // Diagnostic output
     switch (Rtn) {
          case 0:
               //UART1_PutQueue(".");
               break;
          case 1:
               UART1_PutQueue("Error\r\n");
               break;
          case 2:
               UART1_PutQueue("Not my IP\r\n");
               break;
          case 3:
               UART1_PutQueue("Not IPV4\r\n");
               break;
          case 4:
               UART1_PutQueue("Unknown packet\r\n");
               break;
     }
*/

     for (i = 0; i < NUM_OF_SOCKET_Intern; i++) {
          if (socket_Intern[i].open == 0)
              pos[i] = 0;
     }
}

/*************************************************************************************************/
// UDP Callback Function
/*************************************************************************************************/
unsigned int Net_Ethernet_Intern_UserUDP(UDP_Intern_Dsc *udpDsc) {
     unsigned int len;      // My reply length
     unsigned int i;

     int Year, Month, Day, DoW, Hour, Minute, Second;
     char String[8];
     char *BufNum;
     
     len = 0;  // Initialize len

     // UDP Server (Responds to UDP Utility) ******************************************************
     
     if (udpDsc->destPort == 6060) {  // We will listen for messages on this port
         
         if (udpDsc->dataLength < 60) {  // To prevent buffer overruns
             
             for (i=0; i<udpDsc->dataLength; i++) {  // Copy internal buffer
                  UDP_Buffer[i] = Net_Ethernet_Intern_getByte();
             }
             UDP_Buffer[i] = 0;  // Add NUL character to end of buffer
             
             switch (UDP_Buffer[0]) {  // First character is the command
                  
                  case 'A':  // Show Date/Time
                     UART1_PutQueue("A command received\r\n");
                     Year = ts1.yy;    // MikroE sprintf has trouble with char type
                     Month = ts1.mo;   // so put values into integer type
                     Day = ts1.md;
                     Hour = ts1.hh;
                     Minute = ts1.mn;
                     Second = ts1.ss;
                     
                     // Format the date/time and send it back to the client
                     sprintf(UDP_Buffer,"%04d/%02d/%02d %02d:%02d:%02d\r\n",Year,Month,Day,Hour,Minute,Second);
                     len = strlen(UDP_Buffer);
                     Net_Ethernet_Intern_putBytes(UDP_Buffer,len);
                     break;
                  
                  case 'B':  // Set Date/Time
                     UART1_PutQueue("B command received\r\n");
                     if (strlen(UDP_Buffer) == 24) {  // Quick check of buffer length
                         UART1_PutQueue("Setting date/time \r\n");
                         String[2] = 0;

                         String[0] = UDP_Buffer[2];
                         String[1] = UDP_Buffer[3];
                         ts1.yy = 2000 + atoi(String);   // Year

                         String[0] = UDP_Buffer[5];
                         String[1] = UDP_Buffer[6];
                         ts1.mo = atoi(String);          // Month

                         String[0] = UDP_Buffer[8];
                         String[1] = UDP_Buffer[9];
                         ts1.md = atoi(String);          // Day of month

                         String[0] = UDP_Buffer[11];
                         String[1] = UDP_Buffer[12];
                         Dow = atoi(String);             // Day of week
                         
                         switch (DoW) {                  // Translate between Delphi7 and mikroC
                             case 1: ts1.wd = 6; break;  // Sunday
                             case 2: ts1.wd = 0; break;  // Monday
                             case 3: ts1.wd = 1; break;  // Tuesday
                             case 4: ts1.wd = 2; break;  // Wednesday
                             case 5: ts1.wd = 3; break;  // Thursday
                             case 6: ts1.wd = 4; break;  // Friday
                             case 7: ts1.wd = 5; break;  // Saturday
                         }

                         String[0] = UDP_Buffer[14];
                         String[1] = UDP_Buffer[15];
                         ts1.hh = atoi(String);          // Hours

                         String[0] = UDP_Buffer[17];
                         String[1] = UDP_Buffer[18];
                         ts1.mn = atoi(String);          // Minutes

                         String[0] = UDP_Buffer[20];
                         String[1] = UDP_Buffer[21];
                         ts1.ss = atoi(String);          // Seconds

                         Epoch = Time_dateToEpoch(&ts1);
                     
                         // Send "Date/Time set" to client to complete the transaction
                         len = strlen(UDP_Message1);
                         Net_Ethernet_Intern_putBytes(UDP_Message1,len);
                     
                     } else {
                         BufNum = GetBuffer();
                         sprintf(BufNum,"Incorrect length: %d \r\n",strlen(UDP_Buffer));
                         UART1_PutQueue(BufNum);
                         
                         // Send "NAK" to client to complete the transaction
                         len = strlen(UDP_Message2);
                         Net_Ethernet_Intern_putBytes(UDP_Message2,len);
                     }
                     break;
                     
                  default:
                     UART1_PutQueue("Unknown command\r\n");  // Diagnostic message
                     break;
             }

         } else {
             UART1_PutQueue("UDP message length exceeded\r\n");  // Diagnostic message
         }
     
         BufNum = GetBuffer();
         sprintf(BufNum,"Remote Port: %u  Destination Port: %u\r\n",udpDsc->remotePort,udpDsc->destPort);
         UART1_PutQueue(BufNum);
     
     }

     // SNTP server *******************************************************************************
     if (udpDsc->destPort == 123) {  // SNTP communicates on this port

         SNTPTransmitFrame.ReceiveTS1 = Epoch-Fudge;  // Seconds since 1900
         // http://thompsonng.blogspot.ca/2010/04/ntp-timestamp_21.html
         SNTPTransmitFrame.ReceiveTS2 = (msec * 4294967296) / 1000; // SNTP Fraction = (Value in milliseconds * 2^32) / 10^3
         

         SNTPReceiveFrame.Flags = Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.Flags <<= 8;
         SNTPReceiveFrame.Flags |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.Flags <<= 8;
         SNTPReceiveFrame.Flags |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.Flags <<= 8;
         SNTPReceiveFrame.Flags |= Net_Ethernet_Intern_getByte();
         
         BufNum = GetBuffer();
         sprintf(BufNum,"Flags: %08LX\r\n",SNTPReceiveFrame.Flags);
         UART1_PutQueue(BufNum);
     
         SNTPReceiveFrame.RootDelay = Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.RootDelay <<= 8;
         SNTPReceiveFrame.RootDelay |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.RootDelay <<= 8;
         SNTPReceiveFrame.RootDelay |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.RootDelay <<= 8;
         SNTPReceiveFrame.RootDelay |= Net_Ethernet_Intern_getByte();
         
         BufNum = GetBuffer();
         sprintf(BufNum,"RootDelay: %08LX\r\n",SNTPReceiveFrame.RootDelay);
         UART1_PutQueue(BufNum);

         SNTPReceiveFrame.RootDispersion = Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.RootDispersion <<= 8;
         SNTPReceiveFrame.RootDispersion |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.RootDispersion <<= 8;
         SNTPReceiveFrame.RootDispersion |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.RootDispersion <<= 8;
         SNTPReceiveFrame.RootDispersion |= Net_Ethernet_Intern_getByte();

         BufNum = GetBuffer();
         sprintf(BufNum,"RootDispersion: %08LX\r\n",SNTPReceiveFrame.RootDispersion);
         UART1_PutQueue(BufNum);

         SNTPReceiveFrame.ReferenceID = Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.ReferenceID <<= 8;
         SNTPReceiveFrame.ReferenceID |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.ReferenceID <<= 8;
         SNTPReceiveFrame.ReferenceID |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.ReferenceID <<= 8;
         SNTPReceiveFrame.ReferenceID |= Net_Ethernet_Intern_getByte();

         BufNum = GetBuffer();
         sprintf(BufNum,"ReferenceID: %08LX\r\n",SNTPReceiveFrame.ReferenceID);
         UART1_PutQueue(BufNum);

         SNTPReceiveFrame.RefTS1 = Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.RefTS1 <<= 8;
         SNTPReceiveFrame.RefTS1 |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.RefTS1 <<= 8;
         SNTPReceiveFrame.RefTS1 |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.RefTS1 <<= 8;
         SNTPReceiveFrame.RefTS1 |= Net_Ethernet_Intern_getByte();

         BufNum = GetBuffer();
         sprintf(BufNum,"RefTS1: %08LX\r\n",SNTPReceiveFrame.RefTS1);
         UART1_PutQueue(BufNum);

         SNTPReceiveFrame.RefTS2 = Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.RefTS2 <<= 8;
         SNTPReceiveFrame.RefTS2 |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.RefTS2 <<= 8;
         SNTPReceiveFrame.RefTS2 |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.RefTS2 <<= 8;
         SNTPReceiveFrame.RefTS2 |= Net_Ethernet_Intern_getByte();

         BufNum = GetBuffer();
         sprintf(BufNum,"RefTS2: %08LX\r\n",SNTPReceiveFrame.RefTS2);
         UART1_PutQueue(BufNum);

         SNTPReceiveFrame.OriginTS1 = Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.OriginTS1 <<= 8;
         SNTPReceiveFrame.OriginTS1 |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.OriginTS1 <<= 8;
         SNTPReceiveFrame.OriginTS1 |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.OriginTS1 <<= 8;
         SNTPReceiveFrame.OriginTS1 |= Net_Ethernet_Intern_getByte();

         BufNum = GetBuffer();
         sprintf(BufNum,"OriginTS1: %08LX\r\n",SNTPReceiveFrame.OriginTS1);
         UART1_PutQueue(BufNum);

         SNTPReceiveFrame.OriginTS2 = Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.OriginTS2 <<= 8;
         SNTPReceiveFrame.OriginTS2 |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.OriginTS2 <<= 8;
         SNTPReceiveFrame.OriginTS2 |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.OriginTS2 <<= 8;
         SNTPReceiveFrame.OriginTS2 |= Net_Ethernet_Intern_getByte();

         BufNum = GetBuffer();
         sprintf(BufNum,"OriginTS2: %08LX\r\n",SNTPReceiveFrame.OriginTS2);
         UART1_PutQueue(BufNum);

         SNTPReceiveFrame.ReceiveTS1 = Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.ReceiveTS1 <<= 8;
         SNTPReceiveFrame.ReceiveTS1 |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.ReceiveTS1 <<= 8;
         SNTPReceiveFrame.ReceiveTS1 |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.ReceiveTS1 <<= 8;
         SNTPReceiveFrame.ReceiveTS1 |= Net_Ethernet_Intern_getByte();

         BufNum = GetBuffer();
         sprintf(BufNum,"ReceiveTS1: %08LX\r\n",SNTPReceiveFrame.ReceiveTS1);
         UART1_PutQueue(BufNum);

         SNTPReceiveFrame.ReceiveTS2 = Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.ReceiveTS2 <<= 8;
         SNTPReceiveFrame.ReceiveTS2 |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.ReceiveTS2 <<= 8;
         SNTPReceiveFrame.ReceiveTS2 |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.ReceiveTS2 <<= 8;
         SNTPReceiveFrame.ReceiveTS2 |= Net_Ethernet_Intern_getByte();

         BufNum = GetBuffer();
         sprintf(BufNum,"ReceiveTS2: %08LX\r\n",SNTPReceiveFrame.ReceiveTS2);
         UART1_PutQueue(BufNum);

         SNTPReceiveFrame.TransmitTS1 = Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.TransmitTS1 <<= 8;
         SNTPReceiveFrame.TransmitTS1 |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.TransmitTS1 <<= 8;
         SNTPReceiveFrame.TransmitTS1 |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.TransmitTS1 <<= 8;
         SNTPReceiveFrame.TransmitTS1 |= Net_Ethernet_Intern_getByte();

         BufNum = GetBuffer();
         sprintf(BufNum,"TransmitTS1: %08LX\r\n",SNTPReceiveFrame.TransmitTS1);
         UART1_PutQueue(BufNum);

         SNTPReceiveFrame.TransmitTS2 = Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.TransmitTS2 <<= 8;
         SNTPReceiveFrame.TransmitTS2 |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.TransmitTS2 <<= 8;
         SNTPReceiveFrame.TransmitTS2 |= Net_Ethernet_Intern_getByte();
         SNTPReceiveFrame.TransmitTS2 <<= 8;
         SNTPReceiveFrame.TransmitTS2 |= Net_Ethernet_Intern_getByte();

         BufNum = GetBuffer();
         sprintf(BufNum,"TransmitTS2: %08LX\r\n\n",SNTPReceiveFrame.TransmitTS2);
         UART1_PutQueue(BufNum);

         SNTPTransmitFrame.Flags = 0xDC000AFA;
         SNTPTransmitFrame.RootDelay = 0;
         SNTPTransmitFrame.RootDispersion = 0x00010290;
         //SNTPTransmitFrame.ReferenceID = 0x57575642;
         SNTPTransmitFrame.ReferenceID = 0;
         SNTPTransmitFrame.RefTS1 = 0;
         SNTPTransmitFrame.RefTS2 = 0;
         SNTPTransmitFrame.OriginTS1 = SNTPReceiveFrame.TransmitTS1;
         SNTPTransmitFrame.OriginTS2 = SNTPReceiveFrame.TransmitTS2;
         // ReceiveTS1 already done above (time of request arival)
         // ReceiveTS2 already done above
         SNTPTransmitFrame.TransmitTS1 = Epoch-Fudge;  // Seconds since 1900
         
         // http://thompsonng.blogspot.ca/2010/04/ntp-timestamp_21.html
         SNTPTransmitFrame.TransmitTS2 = (msec * 4294967296) / 1000; // SNTP Fraction = (Value in milliseconds * 2^32) / 10^3


         SNTP_TxBuffer[0] = (SNTPTransmitFrame.Flags >> 24);
         SNTP_TxBuffer[1] = (SNTPTransmitFrame.Flags >> 16);
         SNTP_TxBuffer[2] = (SNTPTransmitFrame.Flags >> 8);
         SNTP_TxBuffer[3] = (SNTPTransmitFrame.Flags);

         SNTP_TxBuffer[4] = (SNTPTransmitFrame.RootDelay >> 24);
         SNTP_TxBuffer[5] = (SNTPTransmitFrame.RootDelay >> 16);
         SNTP_TxBuffer[6] = (SNTPTransmitFrame.RootDelay >> 8);
         SNTP_TxBuffer[7] = (SNTPTransmitFrame.RootDelay);

         SNTP_TxBuffer[8] = (SNTPTransmitFrame.RootDispersion >> 24);
         SNTP_TxBuffer[9] = (SNTPTransmitFrame.RootDispersion >> 16);
         SNTP_TxBuffer[10] = (SNTPTransmitFrame.RootDispersion >> 8);
         SNTP_TxBuffer[11] = (SNTPTransmitFrame.RootDispersion);

         SNTP_TxBuffer[12] = (SNTPTransmitFrame.ReferenceID >> 24);
         SNTP_TxBuffer[13] = (SNTPTransmitFrame.ReferenceID >> 16);
         SNTP_TxBuffer[14] = (SNTPTransmitFrame.ReferenceID >> 8);
         SNTP_TxBuffer[15] = (SNTPTransmitFrame.ReferenceID);

         SNTP_TxBuffer[16] = (SNTPTransmitFrame.RefTS1 >> 24);
         SNTP_TxBuffer[17] = (SNTPTransmitFrame.RefTS1 >> 16);
         SNTP_TxBuffer[18] = (SNTPTransmitFrame.RefTS1 >> 8);
         SNTP_TxBuffer[19] = (SNTPTransmitFrame.RefTS1);

         SNTP_TxBuffer[20] = (SNTPTransmitFrame.RefTS2 >> 24);
         SNTP_TxBuffer[21] = (SNTPTransmitFrame.RefTS2 >> 16);
         SNTP_TxBuffer[22] = (SNTPTransmitFrame.RefTS2 >> 8);
         SNTP_TxBuffer[23] = (SNTPTransmitFrame.RefTS2);

         SNTP_TxBuffer[24] = (SNTPTransmitFrame.OriginTS1 >> 24);
         SNTP_TxBuffer[25] = (SNTPTransmitFrame.OriginTS1 >> 16);
         SNTP_TxBuffer[26] = (SNTPTransmitFrame.OriginTS1 >> 8);
         SNTP_TxBuffer[27] = (SNTPTransmitFrame.OriginTS1);

         SNTP_TxBuffer[28] = (SNTPTransmitFrame.OriginTS2 >> 24);
         SNTP_TxBuffer[29] = (SNTPTransmitFrame.OriginTS2 >> 16);
         SNTP_TxBuffer[30] = (SNTPTransmitFrame.OriginTS2 >> 8);
         SNTP_TxBuffer[31] = (SNTPTransmitFrame.OriginTS2);

         SNTP_TxBuffer[32] = (SNTPTransmitFrame.ReceiveTS1 >> 24);
         SNTP_TxBuffer[33] = (SNTPTransmitFrame.ReceiveTS1 >> 16);
         SNTP_TxBuffer[34] = (SNTPTransmitFrame.ReceiveTS1 >> 8);
         SNTP_TxBuffer[35] = (SNTPTransmitFrame.ReceiveTS1);

         SNTP_TxBuffer[36] = (SNTPTransmitFrame.ReceiveTS2 >> 24);
         SNTP_TxBuffer[37] = (SNTPTransmitFrame.ReceiveTS2 >> 16);
         SNTP_TxBuffer[38] = (SNTPTransmitFrame.ReceiveTS2 >> 8);
         SNTP_TxBuffer[39] = (SNTPTransmitFrame.ReceiveTS2);

         SNTP_TxBuffer[40] = (SNTPTransmitFrame.TransmitTS1 >> 24);
         SNTP_TxBuffer[41] = (SNTPTransmitFrame.TransmitTS1 >> 16);
         SNTP_TxBuffer[42] = (SNTPTransmitFrame.TransmitTS1 >> 8);
         SNTP_TxBuffer[43] = (SNTPTransmitFrame.TransmitTS1);

         SNTP_TxBuffer[44] = (SNTPTransmitFrame.TransmitTS2 >> 24);
         SNTP_TxBuffer[45] = (SNTPTransmitFrame.TransmitTS2 >> 16);
         SNTP_TxBuffer[46] = (SNTPTransmitFrame.TransmitTS2 >> 8);
         SNTP_TxBuffer[47] = (SNTPTransmitFrame.TransmitTS2);

         BufNum = GetBuffer();
         sprintf(BufNum,"Remote Port: %u  Destination Port: %u\r\n",udpDsc->remotePort,udpDsc->destPort);
         UART1_PutQueue(BufNum);

         len = 48;
         Net_Ethernet_Intern_putBytes(SNTP_TxBuffer,len);
     }

     return(len);
}

/*************************************************************************************************/
// TCP Callback Function
/*************************************************************************************************/
void Net_Ethernet_Intern_UserTCP(SOCKET_Intern_Dsc *socket) {
     unsigned int len;  // Used by HTTP server
     pntr MyPtr;  // Used by TCP client
     unsigned int i;  // Used by TCP server

     // This is the Mikroelektronika HTTP server. Use a browser to access the web page.
     if (socket->destPort == 80) {  // HTTP server listens only to web request on port 80

         // Diagnostic output
         UART1_PutQueue("HTTP request received\r\n");

         // Get 10 first bytes only of the request, the rest does not matter here
         for (len = 0; len < 10; len++) {
              getRequest[len] = Net_Ethernet_Intern_getByte();
         }
         getRequest[len] = 0;

         // Only GET method is supported here
         if (memcmp(getRequest, httpMethod, 5) && (socket->state != 3)) {
             return;
         }

         if (memcmp(getRequest, httpRequ, 9) == 0) {
             sendHTML_mark = 1;
             socketHTML = socket;
         }

         // Check if getRequest contains image request (example: GET /i/p1.jpg):
         if (getRequest[7] == 'p') {
             switch(getRequest[8]) {
                 case '1': {send_mark1 = 1;  socketImage1 = socket; break;}
                 case '2': {send_mark2 = 1;  socketImage2 = socket; break;}
                 case '3': {send_mark3 = 1;  socketImage3 = socket; break;}
                 case '4': {send_mark4 = 1;  socketImage4 = socket; break;}
             }
         }

         //----------------------------------------------------------------------------------------
         // Send picture p1.jpg.
         if ((send_mark1 == 1) && (socketImage1 == socket)) {
             // First put HTTP heder.
             if (pos[socket->ID] == 0)
                 Net_Ethernet_Intern_putConstStringTCP(httpImage, socket);
       
             // Put picture data.
             while (pos[socket->ID] < PICTURE1_SIZE) {
                 // Put byte in TCP Tx buffer. If overflow occurs, do break, and wait for next
                 // call of UserTCP, for this socket.
                 if (Net_Ethernet_Intern_putByteTCP(p1[  pos[socket->ID]++  ], socket) == 0) {
                     // If overflow occurs, decrement pos[socket->ID], because last byte has
                     // not been written into the buffer.
                     pos[socket->ID]--;
                     break;
                 }
             }
             // Close connection if all data sent.
             if (Net_Ethernet_Intern_bufferEmptyTCP(socket) && (pos[socket->ID] >= PICTURE1_SIZE)) {
                 Net_Ethernet_Intern_disconnectTCP(socket);
                 send_mark1 = 0;
                 pos[socket->ID] = 0;
             }
         }

         //----------------------------------------------------------------------------------------
         // Send picture p2.jpg.
         if ((send_mark2 == 1) && (socketImage2 == socket)) {
             if (pos[socket->ID] == 0)
                 Net_Ethernet_Intern_putConstStringTCP(httpImage, socket);

             while (pos[socket->ID] < PICTURE2_SIZE) {
                 if (Net_Ethernet_Intern_putByteTCP(p2[pos[socket->ID]++], socket) == 0) {
                     pos[socket->ID]--;
                     break;
                 }
             }
             if (Net_Ethernet_Intern_bufferEmptyTCP(socket) && (pos[socket->ID] >= PICTURE2_SIZE)) {
                 Net_Ethernet_Intern_disconnectTCP(socket);
                 send_mark2 = 0;
                 pos[socket->ID] = 0;
             }
         }

         //----------------------------------------------------------------------------------------
         // Send picture p3.jpg.
         if ((send_mark3 == 1) && (socketImage3 == socket)) {
             if (pos[socket->ID] == 0)
                 Net_Ethernet_Intern_putConstStringTCP(httpImage, socket);
       
             while (pos[socket->ID] < PICTURE3_SIZE) {
                 if (Net_Ethernet_Intern_putByteTCP(p3[pos[socket->ID]++], socket) == 0) {
                     pos[socket->ID]--;
                     break;
                 }
             }
             if (Net_Ethernet_Intern_bufferEmptyTCP(socket) && (pos[socket->ID] >= PICTURE3_SIZE)) {
                 Net_Ethernet_Intern_disconnectTCP(socket);
                 send_mark3 = 0;
                 pos[socket->ID] = 0;
             }
         }

         //----------------------------------------------------------------------------------------
         // Send picture p4.jpg.
         if ((send_mark4 == 1) && (socketImage4 == socket)){
             if (pos[socket->ID] == 0)
                 Net_Ethernet_Intern_putConstStringTCP(httpImage, socket);

             while (pos[socket->ID] < PICTURE4_SIZE) {
                 if (Net_Ethernet_Intern_putByteTCP(p4[pos[socket->ID]++], socket) == 0) {
                     pos[socket->ID]--;
                     break;
                 }
             }
             if (Net_Ethernet_Intern_bufferEmptyTCP(socket) && (pos[socket->ID] >= PICTURE4_SIZE)) {
                 Net_Ethernet_Intern_disconnectTCP(socket);
                 send_mark4 = 0;
                 pos[socket->ID] = 0;
             }
         }

         //----------------------------------------------------------------------------------------
         // Send picture html page.
         if ((sendHTML_mark == 1) && (socketHTML == socket)) {
             if (pos[socket->ID] == 0) {
                 // Send HTTP header.
                 Net_Ethernet_Intern_putConstStringTCP(httpHeader, socket);
                 Net_Ethernet_Intern_putConstStringTCP(httpMimeTypeHTML, socket);
             }
             while (pos[socket->ID] < HTML_CODE_SIZE) {
                 if (Net_Ethernet_Intern_putByteTCP(html_code[pos[socket->ID]++], socket) == 0) {
                     pos[socket->ID]--;
                     break;
                 }
             }
             if (Net_Ethernet_Intern_bufferEmptyTCP(socket) && (pos[socket->ID] >= HTML_CODE_SIZE)) {
                 Net_Ethernet_Intern_disconnectTCP(socket);
                 sendHTML_mark = 0;
                 pos[socket->ID] = 0;
             }
         }

     /********************************************************************************************/
     // This is the callback routine for the TCP client. Use UniServe for communications.
     } else if (socketTCP_Client == socket) {
         if (FIFO1.TCPC_FIFO_In != FIFO1.TCPC_FIFO_Out) {
             MyPtr = FIFO1.TCPC_FIFO[FIFO1.TCPC_FIFO_Out++];
             if (FIFO1.TCPC_FIFO_Out > (entrysize - 1))
                 FIFO1.TCPC_FIFO_Out = 0;
             Net_Ethernet_Intern_putStringTCP(MyPtr, socketTCP_Client);
         }
         
         if (Net_Ethernet_Intern_bufferEmptyTCP(socketTCP_Client)) {
             Net_Ethernet_Intern_disconnectTCP(socketTCP_Client);
             UART1_PutQueue("TCP Closed\r\n");
         }

     /********************************************************************************************/
     // This is the callback routine for the TCP server.
     } else if (socket->destPort == 1234) {
         socketTCP_Server = socket;
         
         for (i = 0; i < socketTCP_Server->dataLength; i++) {
              userData[i] = Net_Ethernet_Intern_getByte();
         }
         userData[i] = 0;  // Insert NUL into buffer
         UART1_PutQueue(userData);
         Net_Ethernet_Intern_putStringTCP("ACK\r\n", socketTCP_Server);

     } else {
         return;
     }
}

/*************************************************************************************************/
// TCP Client Initialization
void InitTCP_Client() {
     FIFO1.TCPC_FIFO_In = FIFO1.TCPC_FIFO_Out = 0;
     TCPC_ClientState = 0;
}

// TCP client state machine. Call from main loop.
void DoTCP_Client() {

     switch (TCPC_ClientState) {
          
         case TCPC_STATE_WAITING:     // Wait for data in FIFO
             if (FIFO1.TCPC_FIFO_In != FIFO1.TCPC_FIFO_Out) {
                 TCP_Timer = get_timer();  // Get a free timer
                 set_timer(TCP_Timer,4);   // Timeout in 4 seconds
                 TCPC_ClientState = TCPC_STATE_CONNECTING;
                 UART1_PutQueue("TCP Connecting\r\n");
             }
             break;
     
         case TCPC_STATE_CONNECTING:  // Connect to host server
             Net_Ethernet_Intern_connectTCP(rmtIpAddr, rmtPort, locPort, &socketTCP_Client);
             TCPC_ClientState = TCPC_STATE_CHECKING;
             UART1_PutQueue("TCP Checking\r\n");
             break;
     
         case TCPC_STATE_CHECKING:    // Wait to be connected to host server
             if (check_timer(TCP_Timer) != 0) {
                 if (socketTCP_Client->state == 3) {
                     TCPC_ClientState = TCPC_STATE_SENDING;
                     free_timer(TCP_Timer);
                     UART1_PutQueue("TCP Sending\r\n");
                 }
             } else {   // In case server is not running
                 FIFO1.TCPC_FIFO_In = FIFO1.TCPC_FIFO_Out = 0;
                 Net_Ethernet_Intern_disconnectTCP(socketTCP_Client);
                 TCPC_ClientState = TCPC_STATE_WAITING;
                 free_timer(TCP_Timer);
                 UART1_PutQueue("TCP Waiting\r\n");
             }
             break;
         
         case TCPC_STATE_SENDING:     // Send data to host server
              Net_Ethernet_Intern_startSendTCP(socketTCP_Client);
              TCPC_ClientState = TCPC_STATE_CLOSING;
              UART1_PutQueue("TCP Closing\r\n");
             break;
     
         case TCPC_STATE_CLOSING:     // Wait for connection to close
              if (socketTCP_Client->state == 0) {
                  TCPC_ClientState = TCPC_STATE_WAITING;
                  UART1_PutQueue("TCP Waiting\r\n");
              }
             break;
     }
}

// Makes a message pointer entry to TCP FIFO. Call to put message pointer into TCP client FIFO.
void Put_FIFO1(char *msg) {
     FIFO1.TCPC_FIFO[FIFO1.TCPC_FIFO_In++] = msg;
     if (FIFO1.TCPC_FIFO_In > (entrysize - 1))
         FIFO1.TCPC_FIFO_In = 0;
}

