/******************************************************************************
File:         Timers.c
Description:  Implements simple timer pool. No timer chaining
Resolution:   +0/-0.999 second
Version Info:
*******************************************************************************/
#include "timers.h"
#include "UART1.h"


// Variables
struct timers TimerPool[NUMTIMERS];



/******************************************************************************
Function:   init_timerpool

Parameters: none

Return val: void

Date:       2012/12/26

Desc:       Initialize timer pool
*******************************************************************************/
void init_timerpool(void)
{
     unsigned short int i;

     for (i=0; i < NUMTIMERS; i++)
     {
          TimerPool[i].TimerField = 0;
          TimerPool[i].Free = TRUE;
     }
}


/******************************************************************************
Function:   get_timer

Parameters: none

Return val: unsigned short int - returns handle of free timer

Date:       2012/12/26

Desc:       Gets free timer from the timer pool
*******************************************************************************/
unsigned short int get_timer(void)
{
     unsigned short int i;
     unsigned short int FirstMatch;

     for (i=0; i < NUMTIMERS; i++)
     {
          if (TimerPool[i].Free == TRUE)
          {
              // We found a free timer!
              // Mark it reserved
              TimerPool[i].Free = FALSE;
              FirstMatch = i;
              return FirstMatch;  // Return Handle
          }
     }

     // Error Check
     UART1_PutQueue("TIMERS: No Timers, Resetting..\r\n");
     //RESET_SYSTEM();
     return (0);
}


/******************************************************************************
Function:   free_timer

Parameters: none

Return val: unsigned short int nbr - number

Date:       2012/12/26

Desc:       Puts the given timer back to the timer pool
*******************************************************************************/
void free_timer(unsigned short int nbr)
{
     // Make a simple check
     if (nbr > (NUMTIMERS-1))
         return;

     TimerPool[nbr].Free = TRUE;
}


/******************************************************************************
Function:   set_timer

Parameters: unsigned short int nbr - number
            unsigned long int tout - time out

Return val: none

Date:       2012/12/26

Desc:       Initializes given timer to given time-out
*******************************************************************************/
void set_timer(unsigned short int nbr, unsigned long int tout)
{
     unsigned long int val;

     // Make a simple check
     if (nbr > (NUMTIMERS-1))
         return;

     if (TimerPool[nbr].Free == TRUE)
         return;

     // All OK
     val = tout;

     // Normalize seconds to timer tics
     TimerPool[nbr].TimerField = val;
}


/******************************************************************************
Function:   check_timer

Parameters: unsigned short int nbr - number

Return val: unsigned long int - returns the value of given timer

Date:       2012/12/26

Desc:       Return the value of given timer
*******************************************************************************/
unsigned long int check_timer(unsigned short int nbr)
{
     return TimerPool[nbr].TimerField;
}


/******************************************************************************
Function:   irqhandler_timer

Parameters: void

Return val: void

Date:       2012/12/26

Desc:       Call this function on timer interrupt
*******************************************************************************/
void irqhandler_timer(void)
{
     unsigned short int i;

     for (i=0; i<NUMTIMERS; i++)
     {
          if (TimerPool[i].Free == FALSE && TimerPool[i].TimerField != 0)
          TimerPool[i].TimerField--;
     }
}


/* EOF */