/******************************************************************************
File: timers.h

Description: Implements simple timer pool. No timer chaining

Version Info:
*******************************************************************************/

#ifndef TIMERS_H
#define TIMERS_H


// Defines
#define NUMTIMERS 10

#define FALSE 0
#define TRUE 1


//#define TIMERTIC 1000    // Timer period 1/secs

//#define BaseTimer 0      // Free running system timer



struct timers
{
     unsigned long int TimerField;
     unsigned short int Free;
};


/******************************************************************************
Function:   init_timerpool

Parameters: none

Return val: void

Date:       2012/12/26

Desc:       Initialize timer pool
*******************************************************************************/
void init_timerpool(void);



/******************************************************************************
Function:   get_timer

Parameters: none

Return val: Return handle of free timer

Date:       2012/12/26

Desc:       Gets free timer from the timer pool
*******************************************************************************/
unsigned short int get_timer(void);


/******************************************************************************
Function:   free_timer

Parameters: none

Return val: unsigned short int nbr - number

Date:       2012/12/26

Desc:       Puts the given timer back to the timer pool
*******************************************************************************/
void free_timer(unsigned short int nbr);


/******************************************************************************
Function:   set_timer

Parameters: unsigned short int nbr - number
            unsigned long int tout - time out

Return val: none

Date:       2012/12/26

Desc:       Initializes given timer to given time-out
*******************************************************************************/
void set_timer(unsigned short int nbr, unsigned long int tout);


/******************************************************************************
Function:   check_timer

Parameters: unsigned short int nbr - number

Return val: unsigned long int - returns the value of given timer

Date:       2012/12/26

Desc:       Return the value of given timer
*******************************************************************************/
unsigned long int check_timer(unsigned short int nbr);


/******************************************************************************
Function:   irqhandler_timer

Parameters: void

Return val: void

Date:       2012/12/26

Desc:       Call this function on timer interrupt
*******************************************************************************/
void irqhandler_timer(void);


#endif // guard

/* EOF */